/* Une surcharge de classe ImageView faisant,  l'infini, 
 * appel aux fonctions de traitement de bitmap implmentes en C 
 * 
 * @author Fars BELHADJ (see COPYING) amsi@ai.univ-paris8.fr
 * @date 22 October, 2013
 * */
package com.noalien.graphic2d;

import java.io.PrintStream;
import com.noalien.NDK.g2dn;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.widget.ImageView;

public class MyImageView extends ImageView {
	private Bitmap _bm;
	private int _pixels[];
	private long _t0 = 0, _frames = 0;
	public MyImageView(Context context) {
		super(context);
		_bm = BitmapFactory.decodeResource(getResources(), R.drawable.bg);
		setScaleType(ScaleType.FIT_XY);
		setImageBitmap(_bm);
		_pixels = new int[_bm.getWidth() * _bm.getHeight()];
		initTime();
	}
	@Override
	public void draw(Canvas canvas) {
		super.draw(canvas);
		_bm.getPixels(_pixels, 0, _bm.getWidth(), 0, 0, _bm.getWidth(), _bm.getHeight());
		/* Vous pouvez essayer "la mme" fonction convolution crite en JAVA
		 * Elle est environ 20 fois moins rapide qu'en C !!! 
		 * D'aprs des tests raliss sur l'mulateur pour une image 300x400 - un parcours linaire 
		 * est 6 fois moins rapide en JAVA et une convolution est 20 fois moins rapide */
		//convolute(_pixels, _bm.getWidth(), _bm.getHeight());
		g2dn.convolute(_pixels, _bm.getWidth(), _bm.getHeight());
		_bm.setPixels(_pixels, 0, _bm.getWidth(), 0, 0, _bm.getWidth(), _bm.getHeight());		
		invalidate();
		printFPS(System.out);
	}
	public void initTime() {
		_t0 = System.currentTimeMillis();
		_frames = 0;
	}
	public void printFPS(PrintStream ps) {
		long t = System.currentTimeMillis();
		if(t - _t0 > 5000) {
			ps.println("FPS: " + (_frames / ((t - _t0) / 1000f)));
			ps.flush();
			_t0 = t;
			_frames = 0;
		}
		_frames++;
	}
	static public void convolute(int pixels[], int pw, int ph) {
		int x, y, ypw, r, g, b, moi, voisinH, voisinB, voisinG, voisinD;
		int temp[] = new int[pw * ph];
		for(y = 1; y < ph - 1; y++) {
			for(x = 1; x < pw - 1; x++) {
				moi     = pixels[(ypw = y * pw) + x];
				voisinG = pixels[ypw + x - 1];
				voisinD = pixels[ypw + x + 1];
				voisinH = pixels[ypw - pw + x];
				voisinB = pixels[ypw + pw + x];
				r = (Color.red(voisinH) + Color.red(voisinB) + Color.red(voisinG) + Color.red(voisinD) + Color.red(moi)) / 5;
				g = (Color.green(voisinH) + Color.green(voisinB) + Color.green(voisinG) + Color.green(voisinD) + Color.green(moi)) / 5;
				b = (Color.blue(voisinH) + Color.blue(voisinB) + Color.blue(voisinG) + Color.blue(voisinD) + Color.blue(moi)) / 5;
				temp[y * pw + x] = Color.argb(255, r, g, b);
			}
		}
		System.arraycopy(temp, 0, pixels, 0, pixels.length);
	}
}
